
from qserious import deserialize

result_1 = {
    'hola': 'caracola',
    'adios': 'caracol',
    'foo': 'bar',
    'nested': {'value': 'foobar'}
}
result_2 = {
    'origin': {
        'url': 'https://www.example.com/item/item_id'
    }
}


def test_dots():
    qs = r'hola=caracola&adios=caracol&foo=bar&nested.value=foobar'
    assert deserialize(qs) == result_1


def test_brackets():
    qs = r'hola=caracola&adios=caracol&foo=bar&nested[value]=foobar'
    assert deserialize(qs) == result_1


def test_brackets_start():
    qs = r'hola=caracola&adios=caracol&foo=bar&[nested]value=foobar'
    assert deserialize(qs) == result_1


def test_url():
    qs = r'origin[url]=https%3A%2F%2Fwww.example.com%2Fitem%2Fitem_id'
    result = deserialize(qs)
    assert result == {
        'origin': {'url': 'https://www.example.com/item/item_id'}
    }




from qserious import serialize, deserialize

data_1 = {
    'hola': 'caracola',
    'adios': 'caracol',
    'foo': 'bar',
    'nested': {
        'value': 'foobar'
    }
}


def test_data1_1():
    espected = 'nested[value]=foobar&foo=bar&hola=caracola&adios=caracol'
    assert serialize(data_1) == espected


def test_data1_2():
    assert deserialize(serialize(data_1)) == data_1


def test_data2():
    data = {'nested': {'value': 'foobar'}}
    expected = 'nested[value]=foobar'
    assert serialize(data) == expected



# QueryString deserializer

import re
import urllib


def setKey(o, keys, value):
    if(len(keys) > 1):
        o[keys[0]] = {}
        setKey(o[keys[0]], keys[1:], value)
    else:
        o[keys[0]] = value


def bracketsToDots(text):
    return re.sub(
        r"\]\[|\[|\]", ".",
        re.sub(
            r"\]$", "",
            re.sub(r"^\[", "", text)
        )
    )


def deserialize(qs):
    if(not isinstance(qs, str)):
        raise TypeError('should be a string')

    params = qs.split('&')
    result = {}

    for param in params:
        parts = param.split('=')
        setKey(
            result,
            bracketsToDots(parts[0]).split('.'),
            urllib.unquote(parts[1])
        )

    return result


def keysTobrackets(keys):
    result = ''
    for i, key in enumerate(keys):
        if i > 0:
            result += '[' + key + ']'
        else:
            result += key
    return result


def _serialize(data, params, keys):
    iterable = False
    try:
        iter(data)
        iterable = True
    except TypeError:
        pass

    if not iterable or isinstance(data, str):
        params.append(
            keysTobrackets(keys) + '=' + urllib.quote(
                str(data),
                safe='~()*!.\''
            )
        )
        return

    for k in data:
        _keys = keys[:]
        _keys.append(k)
        _serialize(data[k], params, _keys)


def serialize(data):
    params = []
    _serialize(data, params, [])
    return '&'.join(params)
